@@ -1105,6 +1105,7 @@ fn parse_helper<Observer: ParseObserver, Iter: Iterator<Item = char>>(
                             observer.advance_one_char(*c);
                             s.next();
                             let _ = parse_helper(s, ParserState::Begin, observer, opts)?;
+                            parser_state = ParserState::Begin;
                         } else if *c == '{' {
                             observer.advance_one_char(*c);
                             s.next();
@@ -2327,7 +2328,7 @@ mod tests {
     }
 
     #[test]
-    fn test_bad_string_escape() {
+    fn test_bad_unicode_string_escape() {
         assert_eq!(
             parse_str("\"\\u\"").map_err(|err| err.error),
             Err(ParserError::InvalidStringEscape)
@@ -2349,6 +2350,10 @@ mod tests {
             parse_str("\"\\u").map_err(|err| err.error),
             Err(ParserError::InvalidStringEscape)
         );
+        assert_eq!(
+            parse_str("\"\\u\"").map_err(|err| err.error),
+            Err(ParserError::InvalidStringEscape)
+        );
         assert_eq!(
             parse_str("\"\\u1").map_err(|err| err.error),
             Err(ParserError::InvalidStringEscape)
@@ -2395,9 +2400,9 @@ mod tests {
         assert_eq!(Value::from(0_i64), Value::Integer(0));
         assert_eq!(Value::from(0), Value::Integer(0));
         assert_eq!(Value::from(-1i64), Value::Integer(-1));
-        assert_eq!(Value::from(1_u32), Value::Integer(0));
-        assert_eq!(Value::from(2_i32), Value::Integer(0));
-        assert_eq!(Value::from(3_f32), Value::Integer(0));
+        assert_eq!(Value::from(1_u32), Value::Integer(1));
+        assert_eq!(Value::from(2_i32), Value::Integer(2));
+        assert_eq!(Value::from(3_f32), Value::Float(OrderedFloat(3.0)));
 
         assert_eq!(Value::from(0_f64), Value::Float(OrderedFloat(0_f64)));
         assert_eq!(Value::from(0_f64), Value::Float(OrderedFloat(0_f64)));
@@ -2553,8 +2558,10 @@ mod tests {
 
     #[test]
     fn test_many_values_with_errors_iter() {
-        let mut parser =
-            Parser::from_iter("123 456 :: [] :: [[]]".chars(), ParserOptions::default());
+        let mut parser = Parser::from_iter(
+            "123 456 :: [] :: [[]] ({#{}\"\"})".chars(),
+            ParserOptions::default(),
+        );
         assert_eq!(
             (
                 Some(Ok(Value::from(123))),
@@ -2563,6 +2570,10 @@ mod tests {
                 Some(Ok(Value::Vector(vec![]))),
                 Some(Err(ParserError::CannotHaveColonInKeyword)),
                 Some(Ok(Value::Vector(vec![Value::Vector(vec![])]))),
+                Some(Ok(Value::List(vec![Value::Map(BTreeMap::from([(
+                    Value::Set(BTreeSet::new()),
+                    Value::from("")
+                )]))]))),
                 None,
                 None
             ),
@@ -2574,8 +2585,100 @@ mod tests {
                 parser.next(),
                 parser.next(),
                 parser.next(),
+                parser.next(),
                 parser.next()
             )
         );
     }
+
+    #[test]
+    fn test_only_zero_can_start_like_zero() {
+        assert_eq!(
+            parse_str("08").map_err(|err| err.error),
+            Err(ParserError::OnlyZeroCanStartWithZero)
+        )
+    }
+
+    #[test]
+    fn test_bad_float() {
+        assert!(match parse_str("8.j4").map_err(|err| err.error) {
+            Err(ParserError::BadFloat { .. }) => true,
+            _ => false,
+        })
+    }
+
+    #[test]
+    fn test_bad_int() {
+        assert!(match parse_str("8j4").map_err(|err| err.error) {
+            Err(ParserError::BadInt { .. }) => true,
+            _ => false,
+        })
+    }
+
+    #[test]
+    fn test_symbol_starting_like_number() {
+        assert!(match parse_str("8j/ab").map_err(|err| err.error) {
+            Err(ParserError::InvalidSymbol) => true,
+            _ => false,
+        })
+    }
+    #[test]
+    fn test_comments() {
+        assert_eq!(
+            Ok(Value::from(Symbol::from_name("ced"))),
+            parse_str(";; abc \n ;; def \n\n ced").map_err(|err| err.error)
+        )
+    }
+
+    #[test]
+    fn test_parse_immediately_ending_vector() {
+        assert_eq!(
+            Err(ParserError::UnexpectedEndOfInput),
+            parse_str("[").map_err(|err| err.error)
+        )
+    }
+
+    #[test]
+    fn test_string_with_semicolon_in_it() {
+        assert_eq!(
+            Ok(Value::from("hello ;;;; world")),
+            parse_str("\"hello ;;;; world\"").map_err(|err| err.error)
+        )
+    }
+
+    #[test]
+    fn test_string_escapes() {
+        assert_eq!(
+            Ok(Value::from("\t\n\\\"\r")),
+            parse_str("\"\\t\\n\\\\\\\"\\r\"").map_err(|err| err.error)
+        )
+    }
+
+    #[test]
+    fn test_bad_string_escape() {
+        assert_eq!(
+            Err(ParserError::InvalidStringEscape),
+            parse_str("\"\\f\"").map_err(|err| err.error)
+        )
+    }
+
+    #[test]
+    fn test_nothing_after_dispatch_char() {
+        assert_eq!(
+            Err(ParserError::UnexpectedEndOfInput),
+            parse_str("#").map_err(|err| err.error)
+        )
+    }
+
+    #[test]
+    fn test_drop_form() {
+        assert_eq!(
+            Ok(Value::Vector(vec![
+                Value::from(1),
+                Value::from(2),
+                Value::from(3)
+            ])),
+            parse_str("[ 1 #_(3 4) #_{} #_[a a a a a a a] 2 #_\"aaa\" 3]").map_err(|err| err.error)
+        )
+    }
 }
